"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var vscode_languageserver_1 = require("vscode-languageserver");
var vscode_uri_1 = tslib_1.__importDefault(require("vscode-uri"));
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var observable_1 = require("../common/observable");
var util_1 = require("../common/util");
var hunkStream_1 = tslib_1.__importDefault(require("../common/hunkStream"));
var logger_1 = tslib_1.__importDefault(require("../common/logger"));
var securityMap = {
    'error': vscode_languageserver_1.DiagnosticSeverity.Error,
    'warning': vscode_languageserver_1.DiagnosticSeverity.Warning,
    'info': vscode_languageserver_1.DiagnosticSeverity.Information,
    'hint': vscode_languageserver_1.DiagnosticSeverity.Hint
};
var origin$ = new rxjs_1.Subject();
var subscriptions = {};
function sumNum(num) {
    var args = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
    }
    if (num === undefined) {
        return 0;
    }
    return args.reduce(function (res, next) { return res + next; }, parseInt(num, 10));
}
function formatMessage(message, match) {
    return [].concat(message).reduce(function (res, next) {
        if (typeof next === 'number') {
            res += match[next];
        }
        else {
            res += next;
        }
        return res;
    }, '');
}
function getSecurity(securityKey) {
    var security = securityMap[securityKey];
    return security !== undefined ? security : 1;
}
function handleLinter(textDocument, config) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var text, command, _a, rootPatterns, _b, args, sourceName, _c, formatLines, formatPattern, _d, offsetLine, _e, offsetColumn, isStdout, isStderr, _f, securities, diagnostics, workDir, cmd, _g, _h, stdout, _j, stderr, lines, str, m, _k, line, column, message, security, diagnostic, error_1;
        return tslib_1.__generator(this, function (_l) {
            switch (_l.label) {
                case 0:
                    text = textDocument.getText();
                    command = config.command, _a = config.rootPatterns, rootPatterns = _a === void 0 ? [] : _a, _b = config.args, args = _b === void 0 ? [] : _b, sourceName = config.sourceName, _c = config.formatLines, formatLines = _c === void 0 ? 1 : _c, formatPattern = config.formatPattern, _d = config.offsetLine, offsetLine = _d === void 0 ? 0 : _d, _e = config.offsetColumn, offsetColumn = _e === void 0 ? 0 : _e, isStdout = config.isStdout, isStderr = config.isStderr, _f = config.securities, securities = _f === void 0 ? {} : _f;
                    diagnostics = [];
                    // verify params
                    if (!command || !sourceName || !formatLines || !formatPattern) {
                        logger_1.default.error("[" + textDocument.languageId + "] missing config");
                        return [2 /*return*/, diagnostics];
                    }
                    _l.label = 1;
                case 1:
                    _l.trys.push([1, 5, , 6]);
                    return [4 /*yield*/, util_1.findWorkDirectory(vscode_uri_1.default.parse(textDocument.uri).path, rootPatterns)];
                case 2:
                    workDir = _l.sent();
                    return [4 /*yield*/, util_1.findCommand(command, workDir)];
                case 3:
                    cmd = _l.sent();
                    return [4 /*yield*/, util_1.executeFile(new hunkStream_1.default(text), textDocument, cmd, args, {
                            cwd: workDir
                        })];
                case 4:
                    _g = _l.sent(), _h = _g.stdout, stdout = _h === void 0 ? '' : _h, _j = _g.stderr, stderr = _j === void 0 ? '' : _j;
                    lines = [];
                    if (isStdout == undefined && isStderr === undefined) {
                        lines = stdout.split('\n');
                    }
                    else {
                        if (isStdout) {
                            lines = lines.concat(stdout.split('\n'));
                        }
                        if (isStderr) {
                            lines = lines.concat(stderr.split('\n'));
                        }
                    }
                    str = lines.shift();
                    while (lines.length > 0 || str !== undefined) {
                        str = [str].concat(lines.slice(0, formatLines - 1)).join('\n');
                        m = str.match(new RegExp(formatPattern[0]));
                        if (m) {
                            _k = formatPattern[1], line = _k.line, column = _k.column, message = _k.message, security = _k.security;
                            diagnostic = {
                                severity: getSecurity(securities[m[security]]),
                                range: {
                                    start: {
                                        // line and character is base zero so need -1
                                        line: sumNum(m[line], -1, offsetLine),
                                        character: sumNum(m[column], -1, offsetColumn)
                                    },
                                    end: {
                                        line: sumNum(m[line], -1, offsetLine),
                                        character: sumNum(m[column], offsetColumn)
                                    }
                                },
                                message: formatMessage(message, m),
                                source: sourceName
                            };
                            diagnostics.push(diagnostic);
                        }
                        str = lines.shift();
                    }
                    return [3 /*break*/, 6];
                case 5:
                    error_1 = _l.sent();
                    logger_1.default.error("[" + textDocument.languageId + "] diagnostic handle fail: " + error_1.message);
                    return [3 /*break*/, 6];
                case 6: return [2 /*return*/, diagnostics];
            }
        });
    });
}
function handleDiagnostics(textDocument, configs) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var diagnostics, _i, configs_1, linter, dias;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    diagnostics = [];
                    _i = 0, configs_1 = configs;
                    _a.label = 1;
                case 1:
                    if (!(_i < configs_1.length)) return [3 /*break*/, 4];
                    linter = configs_1[_i];
                    return [4 /*yield*/, handleLinter(textDocument, linter)];
                case 2:
                    dias = _a.sent();
                    diagnostics = diagnostics.concat(dias);
                    _a.label = 3;
                case 3:
                    _i++;
                    return [3 /*break*/, 1];
                case 4: return [2 /*return*/, {
                        uri: textDocument.uri,
                        diagnostics: diagnostics
                    }];
            }
        });
    });
}
function next(textDocument, connection, configs) {
    var uri = textDocument.uri;
    if (!subscriptions[uri]) {
        var debounce_1 = Math.max.apply(Math, configs.map(function (i) { return i.debounce; }).concat([100]));
        subscriptions[uri] = origin$.pipe(operators_1.filter(function (textDocument) { return textDocument.uri === uri; }), operators_1.switchMap(function (textDocument) {
            return rxjs_1.timer(debounce_1).pipe(operators_1.map(function () { return textDocument; }));
        }), observable_1.waitMap(function (textDocument) {
            return rxjs_1.from(handleDiagnostics(textDocument, configs));
        })).subscribe(function (diagnostics) {
            connection.sendDiagnostics(diagnostics);
        }, function (error) {
            logger_1.default.error("[" + textDocument.languageId + "]: observable error: " + error.message);
        });
    }
    origin$.next(textDocument);
}
exports.next = next;
function unsubscribe(textDocument) {
    var uri = textDocument.uri;
    var subp = subscriptions[uri];
    if (subp && !subp.closed) {
        subp.unsubscribe();
    }
    subscriptions[uri] = undefined;
}
exports.unsubscribe = unsubscribe;
