"use strict";
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var vscode_languageserver_1 = require("vscode-languageserver");
var handleDiagnostic_1 = require("./handles/handleDiagnostic");
var logger_1 = tslib_1.__importDefault(require("./common/logger"));
var handleFormat_1 = require("./handles/handleFormat");
// create connection by command argv
var connection = vscode_languageserver_1.createConnection();
// init logger
logger_1.default.init(connection);
// sync text document manager
var documents = new vscode_languageserver_1.TextDocuments();
// config of initializationOptions
var config;
// lsp initialize
connection.onInitialize(function (param) {
    var _a = param.initializationOptions, initializationOptions = _a === void 0 ? {} : _a;
    config = initializationOptions;
    return {
        capabilities: {
            textDocumentSync: documents.syncKind,
            documentFormattingProvider: true
        }
    };
});
var handleDiagnostic = function (change) {
    var textDocument = change.document;
    var _a = config.linters, linters = _a === void 0 ? {} : _a, _b = config.filetypes, filetypes = _b === void 0 ? {} : _b;
    if (!filetypes[textDocument.languageId]) {
        return;
    }
    var linter = [].concat(filetypes[textDocument.languageId]);
    var configItems = linter.map(function (l) { return linters[l]; }).filter(function (l) { return l; });
    if (configItems.length === 0) {
        return;
    }
    handleDiagnostic_1.next(textDocument, connection, configItems);
};
// document change or open
documents.onDidChangeContent(handleDiagnostic);
// document will save
documents.onDidSave(handleDiagnostic);
documents.onDidClose(function (evt) {
    handleDiagnostic_1.unsubscribe(evt.document);
});
// listen for document's open/close/change
documents.listen(connection);
// handle format request
connection.onDocumentFormatting(function (params, token) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
    var textDocument, doc, formatters, formatFiletypes, formatterNames, formatterConfigs;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                textDocument = params.textDocument;
                if (!textDocument || !textDocument.uri) {
                    return [2 /*return*/];
                }
                doc = documents.get(textDocument.uri);
                if (!doc) {
                    return [2 /*return*/];
                }
                formatters = config.formatters, formatFiletypes = config.formatFiletypes;
                if (!formatFiletypes[doc.languageId]) {
                    return [2 /*return*/];
                }
                formatterNames = [].concat(formatFiletypes[doc.languageId]);
                formatterConfigs = formatterNames.map(function (n) { return formatters[n]; }).filter(function (n) { return n; });
                if (formatterConfigs.length === 0) {
                    return [2 /*return*/];
                }
                return [4 /*yield*/, handleFormat_1.formatDocument(formatterConfigs, doc, token)];
            case 1: return [2 /*return*/, _a.sent()];
        }
    });
}); });
// lsp start
connection.listen();
