"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var observable_1 = require("./observable");
var handle_1 = require("./handle");
var logger_1 = tslib_1.__importDefault(require("./logger"));
var origin$ = new rxjs_1.Subject();
var subscriptions = {};
function next(textDocument, connection, configs) {
    var uri = textDocument.uri;
    if (!subscriptions[uri]) {
        var debounce_1 = Math.max.apply(Math, configs.map(function (i) { return i.debounce; }).concat([100]));
        subscriptions[uri] = origin$.pipe(operators_1.filter(function (textDocument) { return textDocument.uri === uri; }), operators_1.switchMap(function (textDocument) {
            return rxjs_1.timer(debounce_1).pipe(operators_1.map(function () { return textDocument; }));
        }), observable_1.waitMap(function (textDocument) {
            return rxjs_1.from(handle_1.handleDiagnostics(textDocument, configs));
        })).subscribe(function (diagnostics) {
            connection.sendDiagnostics(diagnostics);
        }, function (error) {
            logger_1.default.error("[" + textDocument.languageId + "]: observable error: " + error.message);
        });
    }
    origin$.next(textDocument);
}
exports.next = next;
function unsubscribe(textDocument) {
    var uri = textDocument.uri;
    var subp = subscriptions[uri];
    if (subp && !subp.closed) {
        subp.unsubscribe();
    }
    subscriptions[uri] = undefined;
}
exports.unsubscribe = unsubscribe;
